-----------------------------------------------------------------------------------------------
-- Client Lua Script for BotBuster
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- BotBuster Module Definition
-----------------------------------------------------------------------------------------------
local BotBuster = {} 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local nDeltat = 0.5 
local nMaxDist = 180
local nMaxSpeed = 100
local tSpeedBuffs = {"Travel Speed Boost", "Jump Boost", "Urgency", "Swiftness", "Road March", "Explorer Trailblazing", "PvP Speed Buff", "Quick Draw", "Combat Speed Boost", "Fleet Like Jabbit Feet", "Safe Fall", "Hot Feet", "Life Mote", "Pressure Release"}
local tGatheringCasts = {"Novice's Laser Pickaxe", "Novice's Laser Chainsaw", "Novice's Relic Blaster"}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function BotBuster:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	
	self.bShouldScan = false	-- changes to false when you're in illium or thayd, scanning there would be pretty useless and resource intesive. No DetectLoop when it's false
	self.bCheckBoxSpeed = true	-- Checkbox boolean that indicates wether notifications will be give or not
	self.bCheckBoxTele = true
	self.tInfo = {}			-- stores all the players that get created and who get pinged every timer loop, holds name, pos1,pos2, dx,dy,dz, speed, dist and detectcount
	self.tDetection = {}   	-- stores the players that triggered a bot detection
   	self.strReportText = nil -- stores the report text that gets made when you want to report a bot
	self.tAlreadyReported = {} -- stores the names of the players who already have gotten reported. Gets stored.
	self.strReportedPlayerName = nil  -- global variable where i can store a name in, that will later get put into the Already Reported table
    return o
end

function BotBuster:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- BotBuster OnLoad
-----------------------------------------------------------------------------------------------
function BotBuster:OnLoad()
    
	self.xmlDoc = XmlDoc.CreateFromFile("BotBuster.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- BotBuster OnDocLoaded
-----------------------------------------------------------------------------------------------


function BotBuster:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    		
		-------------------------MainForm---------------------------------------------------
		self.wndNewMain = Apollo.LoadForm(self.xmlDoc, "MainForm",nil,self)
		self.wndNewMain:Show(false,true)
		
		if self.wndNewMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end

		
		self.WeirdToggleButton = self.wndNewMain:FindChild("WeirdToggle")
		self.AlreadyToggle = self.wndNewMain:FindChild("AlreadyToggle")
		self.OptionsToggleButton = self.wndNewMain:FindChild("OptionsToggle")
		self.BotToggleButton = self.wndNewMain:FindChild("BotToggle")
		

		self.GridTrue2 = self.wndNewMain:FindChild("GridTrue")
		self.GridFalse2 = self.wndNewMain:FindChild("GridFalse")
		self.GridAlready = self.wndNewMain:FindChild("GridAlready")
		self.GridAlready:Show(false)
		self.GridFalse2:Show(false)
		self.BotToggleButton:SetCheck(true)

		-------------------------ReportMenu--------------------------------
		self.wndReportMenu2 = Apollo.LoadForm(self.xmlDoc,"ReportForm",self.wndNewMain,self)
		self.wndReportMenu2:Show(false)
		
		self.ReportButton2 = self.wndReportMenu2:FindChild("Report")

		----------------------Settings Menu---------------------------------		
		self.wndSettingsMenu = Apollo.LoadForm(self.xmlDoc,"SettingsForm",self.wndNewMain,self)
		self.wndSettingsMenu:Show(false)
		
		self.CheckBoxSpeed = self.wndSettingsMenu:FindChild("CheckBox")
		self.CheckBoxTele = self.wndSettingsMenu:FindChild("CheckBox1")
		
		self.CheckBoxSpeed:SetCheck(self.bCheckBoxSpeed)
		self.CheckBoxTele:SetCheck(self.bCheckBoxTele)

		--------------------------Copy Name Menu-----------------------------------
		self.wndCopyNameMenu2 = Apollo.LoadForm(self.xmlDoc,"CopyNameForm",self.wndNewMain,self)
		self.wndCopyNameMenu2:Show(false)
		
		self.CopyNameButton2 = self.wndCopyNameMenu2:FindChild("CopyNameButton")		
				
				
		------------------------------------------------------------------------------------
		
	-- Timers
	self.RefreshTimer = ApolloTimer.Create(nDeltat,true,"OnTimerRefresh",self)	-- This timer handles the main pinging of units that i'm tracking
	
		
	if GameLib.GetPlayerUnit() ~= nil then				-- This loop will handle the charachter creation upon logging in
		self:CheckCurrentZone()
	else		
		Apollo.RegisterEventHandler("CharachterCreated","CheckCurrentZone",self)
	end
	
	Apollo.RegisterEventHandler("SubZoneChanged", "CheckCurrentZone", self)		-- Standard SubZoneChange event to check if we are in Thayd,Illium, Housingworld or PvP zones
	Apollo.RegisterSlashCommand("botbuster","OnBotBuster",self)					-- Slash command used to bring up the main window
	Apollo.RegisterSlashCommand("Testy","OnTesty",self)							-- Slash command i use to test random functions in the API
	Apollo.RegisterEventHandler("UnitCreated","OnUnitCreated",self)
	Apollo.RegisterEventHandler("UnitDestroyed","OnUnitDestroyed",self)
	end
end


-----------------------------------------------------------------------------------------------
-- BotBuster Functions
-----------------------------------------------------------------------------------------------



function BotBuster:CheckCurrentZone() -- Everytime subzone change event happens, we need to check if we are in Thayd/Illium/PvP zone or not
	self:IsInCityHome()
end


function BotBuster:HasSpeedBoost(Unit)		-- Checks if a unit has any speed buffs, bots never have them (they can teleport ... duh)

local tUnitBuffs = Unit:GetBuffs().arBeneficial
local nBuffTableSize = #tSpeedBuffs

	for i,v in pairs(tUnitBuffs) do
		for p=1,nBuffTableSize,1 do
			if tSpeedBuffs[p] == v.splEffect:GetName() then
				return true
			end
		end
	end
	return false
end

--function BotBuster:TableInfoLength()		-- Was created so i could check if i had no memory leaks in my huge tables
--local nLength = 0
--for i,v in pairs(self.tInfo) do
--	nLength = nLength + 1
	--Print(string.format("%s Is still in the table",v.name))
--end

--Print(string.format("Size of tInfo is %d",nLength))
--end



--function BotBuster:OnTesty()				--slash command function where i can test stuff with
--local uTarget = GameLib.GetTargetUnit()
--local strTargetName = uTarget:GetName()
--self.strReportedPlayerName = strTargetName
--Print(self.strReportedPlayerName)
--end




function BotBuster:HasBeenGathering(Unit)	-- Gathering detection, needed for teleport detection. Gets reset every unit creation.
local nGatherTableSize = #tGatheringCasts
	for p=1, nGatherTableSize do
		if tGatheringCasts[p] == Unit:GetCastName() then
			return true
		end
	end
	
	return false
end


function BotBuster:IsInCityHome()		-- Checks wether the player is in Illium, Thayd, Housingworld or PVP zones. 
local nZoneId = GameLib.GetCurrentZoneMap().id
local nIlliumId = GameLib.MapZone.Illium
local nThaydId = GameLib.MapZone.Thayd
local bInHome = HousingLib.IsHousingWorld()
local nWalatikiId = 69
local nHallsId = 99
	if nZoneId == nIlliumId or nZoneId == nThaydId  or nZoneId == nWalatikiId or nZoneId == nHallsId or bInHome then
		self.bShouldScan = false
		for i,v in pairs(self.tInfo) do		
			self.tInfo[i] = nil			-- When we get to thayd or illium or PvP zone reset all the data storage tables besides tDetection.
		end								-- this will free up memory but will keep the names and detections on the list for future use
	else
		self.bShouldScan = true
	end
end






function BotBuster:OnResetClick()		-- Resets the lists and table that stores bot detections, only way to reset these.
self.GridTrue2:DeleteAll()				-- Doesn't reset the "already reported" list
self.GridFalse2:DeleteAll()
	for i,v in pairs(self.tDetection) do
		self.tDetection[i] = nil
	end
end


function BotBuster:OnTimerRefresh()		-- The main ping loop, gets called every 0.5 sec. 
	if self.bShouldScan and GameLib.GetPlayerUnit ~= nil then
		self:DetectionLoop()			-- The magic happens in this function	
	end
end

function BotBuster:OnUnitCreated(Unit)	-- Standard unit creation event. Initializes the table in the tInfo storage for every unit that gets created and has to be tracked.
	if GameLib.GetPlayerUnit() ~= nil then	
		if self.bShouldScan then
			local uPlayer = GameLib.GetPlayerUnit()
			if Unit:IsACharacter()   and Unit ~= uPlayer  and Unit:IsVisibleOnCurrentZoneMinimap()  then -- Last statement is very important to make it lightweight. This will ensure you won't load units in the city Thayd when you are outside for example
				
				local nUnitID = Unit:GetId()
				local strUnitName = Unit:GetName()
				local tPos1 = Unit:GetPosition()
				local tPlayerPos = uPlayer:GetPosition()
				if not self.tInfo[nUnitID] then
					self.tInfo[nUnitID] = {name = strUnitName, pos1 = tPos1, pos2 = 0, dx = 0, dz = 0, speed = 0, dist = math.sqrt(math.pow(tPos1.x - tPlayerPos.x, 2) + math.pow(tPos1.y - tPlayerPos.y, 2) + math.pow(tPos1.z - tPlayerPos.z, 2))
	, detectcount = 1, classid = Unit:GetClassId(), raceid = Unit:GetRaceId(), factionid = Unit:GetFaction(), level = Unit:GetLevel(), gathercount = 0 } 
				end
				--self:TableInfoLength()
				
				--Print(string.format("%s got created",strUnitName))
			end
		end
	end
end



function BotBuster:OnGridRefresh()		-- The grid on the main window has to be refreshed to keep it updated. Will only refresh if the main window is shown.
		self.GridTrue2:DeleteAll()
		self.GridFalse2:DeleteAll()
		self.GridAlready:DeleteAll()
		self:DrawGrid()
		self:DrawAlreadyReportedGrid()
end




function BotBuster:DrawGrid()	 -- This function is  where the sorting happens between weird stuff and bots. The values in the if statements can be changed to make it a lot more strict or a lot more easy to be counted as a bot
	local nRowTrue = 0	
	local nRowFalse = 0
	
	for i,info in pairs(self.tDetection) do
		local strPlayerName = info.name
		local nSpeedCount = info.speedcount
		local nVanishCount = info.vanishcount
		if (nVanishCount > 0 and nSpeedCount > 0 and (nVanishCount + nSpeedCount) > 5) or nSpeedCount > 5 or nVanishCount > 5 then
			nRowTrue = nRowTrue + 1
			self.GridTrue2:AddRow("")
			self.GridTrue2:SetCellText(nRowTrue,1,strPlayerName)
			self.GridTrue2:SetCellText(nRowTrue,3,nVanishCount)
			self.GridTrue2:SetCellText(nRowTrue,2,nSpeedCount)
			self.GridTrue2:SetCellText(nRowTrue,4,i)
		else 
			nRowFalse = nRowFalse+1
			self.GridFalse2:AddRow("")
			self.GridFalse2:SetCellText(nRowFalse,1,strPlayerName)
			self.GridFalse2:SetCellText(nRowFalse,3,nVanishCount)
			self.GridFalse2:SetCellText(nRowFalse,2,nSpeedCount)
		end
	end
end

function BotBuster:OnBotBuster()	-- Shows the main menu and refreshes all the grids
	self.wndNewMain:Show(true)
	self:OnGridRefresh()
end




function BotBuster:DetectionLoop()		-- The function that keeps getting looped to get accurate data on the units that are being tracked. This loop will also track if a unit went too fast but is still in range to calculate the speed
	if GameLib.GetPlayerUnit() ~= nil then
		local uPlayer = GameLib.GetPlayerUnit()
		local tPlayerPos = uPlayer:GetPosition()

		for i,info in pairs(self.tInfo) do
			local uOtherPlayer = GameLib.GetUnitById(i)
			if uOtherPlayer ~= nil then
				if uOtherPlayer:IsDead() then		-- On case of death units do not get destroyed, they just have no life and then get ported to Graveyard. So to combat this i need to reset it like this
					info.pos1 = 0
					info.pos2 = 0
					info.dist = 0
					info.dx = 0
					info.dz = 0
					info.speed = 0
					info.detectcount = 0			
				elseif uOtherPlayer:IsDead() == false and info.detectcount == 0 then -- On ressurection i need to start over from step 1
					if uOtherPlayer:GetFaction() == uPlayer:GetFaction() then
					info.pos1 = uOtherPlayer:GetPosition()
					info.dist = math.sqrt(math.pow(info.pos1.x - tPlayerPos.x, 2) + math.pow(info.pos1.y - tPlayerPos.y, 2) + math.pow(info.pos1.z - tPlayerPos.z, 2))
					end
					info.detectcount = 1
				else		
					info.detectcount = info.detectcount + 1				-- Loop when nobody has died, Only 1 thing for other faction players because position detection is impossible now
					if uOtherPlayer:GetFaction() == uPlayer:GetFaction() then
					info.pos2 = info.pos1
					info.pos1 = uOtherPlayer:GetPosition()
					info.dx = math.abs(info.pos1.x - info.pos2.x)
					info.dz = math.abs(info.pos1.z - info.pos2.z)
					info.dist = math.sqrt(math.pow(info.pos1.x - tPlayerPos.x, 2) + math.pow(info.pos1.y - tPlayerPos.y, 2) + math.pow(info.pos1.z - tPlayerPos.z, 2))
					info.speed = math.floor( math.sqrt( math.pow(info.dx/nDeltat,2) + math.pow(info.dz/nDeltat,2) )+ 0.5)
					end
					--strResult = string.format('%s is going at a speed of %d m/s, and is %d m away',info.name, info.speed, info.dist)
					--Print(strResult)
				end
				
				if self:HasBeenGathering(uOtherPlayer) then		-- Detect if the player has been gathering. Key part for teleport detection
					--Print(string.format("%s is gathering",info.name))
					info.gathercount = info.gathercount + 1
				end
				
				
				if uOtherPlayer:GetFaction() == uPlayer:GetFaction() and self:HasSpeedBoost(uOtherPlayer) == false  and uOtherPlayer:IsDead() == false and uOtherPlayer:IsMounted() == false   and  info.speed > nMaxSpeed and info.speed < 600 and uOtherPlayer:IsInCombat()== false and uOtherPlayer:IsInVehicle() == false then  -- Last statement is needed because bots don't go faster than that but Carbine teleports can cause speeds over 700
			
					
			
					if not self.tDetection[i] then		-- Name gets added to the bot trigger list
						self.tDetection[i] = { name = info.name, speedcount = 0, vanishcount = 0, classid = info.classid, factionid = info.factionid, raceid = info.raceid, level = info.level }
					end
					self.tDetection[i].speedcount = self.tDetection[i].speedcount + 1  -- Number of speed triggers get added to the bot trigger list
			
					if self.bCheckBoxSpeed and self.tDetection[i].speedcount > 1 then					-- Notifications in chat
						Print(string.format('%s was going too fast, at %u m/s'  , info.name, info.speed))
					end
					
					if self.wndNewMain:IsShown() then
						self:OnGridRefresh()
					end
			
				end
			end
		end 
	end
end



function BotBuster:OnUnitDestroyed(Unit)			-- Not the standard unit destruction function. This function is responsible for deleting the unit from the tInfo table when it gets destroyed.
	if self.bShouldScan then
		local uPlayer = GameLib.GetPlayerUnit()
			if Unit:IsACharacter()  and Unit ~= uPlayer and Unit:IsVisibleOnCurrentZoneMinimap() then
				local nUnitID = Unit:GetId()
				local uOtherPlayer = GameLib.GetUnitById(nUnitID)
				local strOtherPlayerName = uOtherPlayer:GetName()
				if self.tInfo[nUnitID] ~= nil then
					if (uOtherPlayer:GetFaction() == uPlayer:GetFaction() and self.tInfo[nUnitID].detectcount < 31 and self.tInfo[nUnitID].gathercount > 0 and self.tInfo[nUnitID].dist < nMaxDist ) or (uOtherPlayer:GetFaction() ~= uPlayer:GetFaction() and  self.tInfo[nUnitID].detectcount < 31 and self.tInfo[nUnitID].gathercount > 0 ) then
			
						if not self.tDetection[nUnitID] then		-- Teleport Detection: If the unit was close to me for a maximum of 30 pings (15s) and has been gathering in those 15s (a little bit longer than 1 node) and he got unit destruction in that small window of time it will trigger, for other faction only sub 15s and gathering matters
							self.tDetection[nUnitID] = {name = strOtherPlayerName, speedcount = 0, vanishcount = 0, classid = self.tInfo[nUnitID].classid,factionid = self.tInfo[nUnitID].factionid, raceid = self.tInfo[nUnitID].raceid, level = self.tInfo[nUnitID].level }
						end
			
						self.tDetection[nUnitID].vanishcount = self.tDetection[nUnitID].vanishcount + 1
						
						if self.bCheckBoxTele and self.tDetection[nUnitID].vanishcount > 1 then		-- chat notification of teleporting
							Print(string.format('%s teleported.',strOtherPlayerName))
						end
						
						
						if self.wndNewMain:IsShown() then
							self:OnGridRefresh()
						end

						
					end	
					
					--Print(string.format("%s got deleted",strOtherPlayerName))
					
				end
				

				self.tInfo[nUnitID] = nil			-- Have to delete the unit out of my table when unit destruction is called on it
				--self:TableInfoLength()
			end
	end
end


-----------------------------------------------------------------------------------------------
-- BotBusterForm Functions
-----------------------------------------------------------------------------------------------

function BotBuster:OnBotButtonToggle()		-- UI stuff follows next
self.BotToggleButton:SetCheck(true)
self.AlreadyToggle:SetCheck(false)
self.WeirdToggleButton:SetCheck(false)
self.GridAlready:Show(false)
self.GridFalse2:Show(false)
self.GridTrue2:Show(true)
end

function BotBuster:OnWeirdButtonToggle()
self.WeirdToggleButton:SetCheck(true)
self.BotToggleButton:SetCheck(false)
self.AlreadyToggle:SetCheck(false)
self.GridAlready:Show(false)
self.GridTrue2:Show(false)
self.GridFalse2:Show(true)
end

function BotBuster:OnAlreadyButtonToggle()
self.AlreadyToggle:SetCheck(true)
self.WeirdToggleButton:SetCheck(false)
self.BotToggleButton:SetCheck(false)
self.GridTrue2:Show(false)
self.GridAlready:Show(true)
self.GridFalse2:Show(false)
end

function BotBuster:OnOptionsToggle()
	if self.wndSettingsMenu:IsShown() then
		self.wndSettingsMenu:Show(false)
		self.OptionsToggleButton:SetCheck(false)
	else
		self.wndSettingsMenu:Show(true)
		self.OptionsToggleButton:SetCheck(true)	
	end
end


-- when the Cancel button is clicked
function BotBuster:OnCancel()
	self:OnBotButtonToggle()
	self.wndNewMain:Close() -- hide the window
	self.wndSettingsMenu:Show(false)
	self.OptionsToggleButton:SetCheck(false)
end




function BotBuster:OnCheckBoxToggle( wndHandler, wndControl, eMouseButton )
	self.bCheckBoxSpeed = self.CheckBoxSpeed:IsChecked()
end

function BotBuster:OnCheckBox1Toggle(wndHandler, wndControl, eMouseButton)
	self.bCheckBoxTele = self.CheckBoxTele:IsChecked()
end


function BotBuster:MakeReportText(nRowNumber)			-- Makes the report text that will be sent to Carbine
	local nUnitID = tonumber(self.GridTrue2:GetCellText(nRowNumber,4))
	
	local strFaction = nil
	local strClass = nil
	local strRace = nil
	
	local nClassID = self.tDetection[nUnitID].classid
	local nRaceID = self.tDetection[nUnitID].raceid
	local nZoneID = GameLib.GetCurrentZoneMap().id
	local nFactionID = self.tDetection[nUnitID].factionid
	
	if nFactionID == Unit.CodeEnumFaction.DominionPlayer then		-- The next 3 if loops can probably be done better
		strFaction = "Dominion"
	else 
		strFaction = "Exile"
	end
	
	if nClassID == GameLib.CodeEnumClass.Engineer then
		strClass = "Engineer"
	elseif nClassID == GameLib.CodeEnumClass.Esper then
		strClass = "Esper"
	elseif nClassID == GameLib.CodeEnumClass.Medic then
		strClass = "Medic"
	elseif nClassID == GameLib.CodeEnumClass.Spellslinger then
		strClass = "Spellslinger"
	elseif nClassID == GameLib.CodeEnumClass.Stalker then
		strClass = "Stalker"
	else
		strClass = "Warrior"
	end
	
	if nRaceID == GameLib.CodeEnumRace.Aurin then
		strRace = "Aurin"
	elseif nRaceID == GameLib.CodeEnumRace.Chua then
		strRace = "Chua"
	elseif nRaceID == GameLib.CodeEnumRace.Draken then
		strRace = "Draken"
	elseif nRaceID == GameLib.CodeEnumRace.Granok then
		strRace = "Granok"
	elseif nRaceID == GameLib.CodeEnumRace.Human then
		strRace = "Human"
	elseif nRaceID == GameLib.CodeEnumRace.Mechari then
		strRace = "Mechari"
	else
		strRace = "Mordesh"
	end
		
	local strPlayerName = self.GridTrue2:GetCellText(nRowNumber,1)
	local strRealmName = GameLib.GetRealmName()
	local strZoneName = GameLib.GetCurrentZoneMap().strName
	local nUnitLvl = self.tDetection[nUnitID].level
	local nSpeedCount = self.GridTrue2:GetCellText(nRowNumber,2)
	local nVanishCount = self.GridTrue2:GetCellText(nRowNumber,3)
	local tLocalTime = GameLib.GetLocalTime()
    local strDate = string.format("Date: %d/%d/%d at %s",tLocalTime.nDay,tLocalTime.nMonth,tLocalTime.nYear,tLocalTime.strFormattedTime)

	
	self.strReportText = string.format(' A Gathering-Bot Report: \n %s \n \n Realm: %s \n Name: %s \n Faction: %s \n Level: %d \n Class: %s \n Race: %s \n Zone: %s \n \n FactionID: %d \n ClassID: %d \n RaceID: %d \n ZoneID: %d \n \n This bot has been detected speeding (going over %d m/s) %d times. \n This bot has been detected teleporting (unit creation and unit destruction in quick succession) %d times. \n \n This is a BotBuster report.',strDate, strRealmName, strPlayerName,strFaction,nUnitLvl,strClass, strRace, strZoneName,nFactionID,nClassID,nRaceID,nZoneID,nMaxSpeed,nSpeedCount,nVanishCount)     
		
end


function BotBuster:OnClick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )	-- Gets called when you click on a name in the bot list
	if self.GridTrue2:GetFocusRow() ~= nil then
		local nRowClick = self.GridTrue2:GetFocusRow()
		local strPlayerName = self.GridTrue2:GetCellText(nRowClick,1)
		self.strReportedPlayerName = strPlayerName
		self:MakeReportText(nRowClick)
		self.ReportButton2:SetText(string.format('Report %s',strPlayerName))
		self.wndReportMenu2:Show(true)
	end
end

function BotBuster:ElementInTable(tTable, eElement)		-- Checks if eElement is in tTable, returns true if it is
	for i,v in pairs(tTable) do
		if v == eElement then
			return true
		end
	end
	return false
end


function BotBuster:DrawAlreadyReportedGrid()			-- Draws the grid that stores the names of the already reported players
	local nSize = #self.tAlreadyReported
	if nSize ~= 0 then
	local nRowNumber = 0
		for i=1,nSize do
			nRowNumber = nRowNumber + 1
			self.GridAlready:AddRow("")
			self.GridAlready:SetCellText(nRowNumber,1,self.tAlreadyReported[i])
		end
	end
end


function BotBuster:OnReportClick()				-- Function that will fetch the right categorie and subcategorie for Bot/ Cheaters and will send the MakeReportText functions output to carbine
local nSize = #self.tAlreadyReported						-- Check if the name is already in the table or not. If it's not, add it.
	if nSize == 0 then
		table.insert(self.tAlreadyReported,self.strReportedPlayerName)
	else
		if not self:ElementInTable(self.tAlreadyReported, self.strReportedPlayerName) then
			table.insert(self.tAlreadyReported,self.strReportedPlayerName)
		end
	end
	
local tListOfEntries = PlayerTicket_GetErrorTypeList()
local nMainCat = 0
local nSubCat = 0
for i,v in ipairs(tListOfEntries) do
	if v.localizedText == "Report Player" then
		nMainCat = i
	end
end

local tListOfSubentries = PlayerTicket_GetSubtype(nMainCat)
for i,v in ipairs(tListOfSubentries) do
	if	v.localizedText == "Bot / Cheater" then
		nSubCat = i
	end
end

	self.GridAlready:DeleteAll()
	self:DrawAlreadyReportedGrid()

	PlayerTicketDialog_Report (nMainCat, nSubCat, self.strReportText)
end



function BotBuster:OnCopyNameButtonClick( wndHandler, wndControl )		-- Copy button for the player in the Weird Behviour list (GridFalse2)
	Print("Name copied to Clipboard. Paste using your standard keycombo.")
end


function BotBuster:OnFalseGridClick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation ) 	-- Gets called when you click on a name in the Weird Behaviour list
	if self.GridFalse2:GetFocusRow() ~= nil then
		local nRowClick = self.GridFalse2:GetFocusRow()
		local strPlayerName = self.GridFalse2:GetCellText(nRowClick,1)
		self.CopyNameButton2:SetActionData(GameLib.CodeEnumConfirmButtonType.CopyToClipboard, strPlayerName)
		self.CopyNameButton2:SetText(string.format('Copy Player Name: %s to Clipboard ?',strPlayerName))
		self.wndCopyNameMenu2:Show(true)
	end
end

function BotBuster:OnSave(eLevel)							-- Saves the notification options and the table of the already reported bots
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

	local save = { bCheckBoxSpeedState = self.bCheckBoxSpeed, bCheckBoxTeleState = self.bCheckBoxTele, tAlreadyReported = self.tAlreadyReported }
	return save
end


function BotBuster:OnRestore(eLevel, tData)			-- Restores the settings for notification and the list of the already reported bots
self.tAlreadyReported = tData.tAlreadyReported or {}
self.bCheckBoxSpeed = tData.bCheckBoxSpeedState
self.bCheckBoxTele = tData.bCheckBoxTeleState
end

-----------------------------------------------------------------------------------------------
-- BotBuster Instance
-----------------------------------------------------------------------------------------------
local BotBusterInst = BotBuster:new()
BotBusterInst:Init()
